#ifndef __CONFIGURATION_H
#define __CONFIGURATION_H

#include <vector>
#include <string>
#include <unistd.h>
#include "json.h"

int32_t _System_PageSize;

std::string  _GeneralSettings_PathRefGenomeFasta;
std::string  _GeneralSettings_PathOutputDir;
std::string  _GeneralSettings_PathBnxFile;
std::string  _GeneralSettings_OutputRFileName;
std::string  _GeneralSettings_OutputRiFileName;
std::string  _GeneralSettings_OutputPosBnxFileName;
std::string  _GeneralSettings_OutputMolIdsBnxFileName;
std::string  _GeneralSettings_MarkerSequence;
std::string  _GeneralSettings_TestResultsOutputFileNamePrefix;
std::string  _GeneralSettings_TestResultsOutputFileNamePostfix;
bool    _GeneralSettings_PrintOutStatistics;
bool    _GeneralSettings_KMersSubstrings;
bool    _GeneralSettings_BnxSubstrings;
bool    _GeneralSettings_BnxReverseMolecules;
int32_t _GeneralSettings_DenoisingDelta;
int32_t _GeneralSettings_BnxRowsToRead;
int32_t _GeneralSettings_BnxMoleculeMinLabelsCount;

int32_t _LengthsEncoderConf_Epsilon;
int32_t _LengthsEncoderConf_LogBase;
float   _LengthsEncoderConf_Power;
float   _LengthsEncoderConf_PolCoeffA;
float   _LengthsEncoderConf_PolCoeffB;
float   _LengthsEncoderConf_PolCoeffC;
float   _LengthsEncoderConf_PolCoeffD;
float   _LengthsEncoderConf_PolCoeffE;

int32_t _GeneralTreeConf_QuestionMark;
int32_t _GeneralTreeConf_Star;
bool    _GeneralTreeConf_ProfilingEnabled;

struct test_definition
{
  std::string _encode_method;
  std::vector<int32_t> _node_stop_creation_depths;
  std::vector<std::string> _tst_forest_key_type;
  int32_t _k_mer_size;
};

std::vector<int16_t> _ChromosomesCnt;
std::vector<std::string> _TestEnable;
std::vector<test_definition> _TestDefinitions;

void read_configuration(nlohmann::json j)
{
  _System_PageSize = getpagesize();

  _GeneralSettings_PathRefGenomeFasta = j["GeneralSettings"]["PathRefGenomeFasta"].get<std::string>();
  _GeneralSettings_PathOutputDir = j["GeneralSettings"]["PathOutputDir"].get<std::string>();
  _GeneralSettings_PathBnxFile = j["GeneralSettings"]["PathBnxFile"].get<std::string>();
  _GeneralSettings_OutputRFileName = j["GeneralSettings"]["OutputRFileName"].get<std::string>();
  _GeneralSettings_OutputRiFileName = j["GeneralSettings"]["OutputRiFileName"].get<std::string>();
  _GeneralSettings_OutputPosBnxFileName = j["GeneralSettings"]["OutputPosBnxFileName"].get<std::string>();
  _GeneralSettings_OutputMolIdsBnxFileName = j["GeneralSettings"]["OutputMolIdsBnxFileName"].get<std::string>();
  _GeneralSettings_MarkerSequence = j["GeneralSettings"]["MarkerSequence"].get<std::string>();
  _GeneralSettings_TestResultsOutputFileNamePrefix = j["GeneralSettings"]["TestResultsOutputFileNamePrefix"].get<std::string>();
  _GeneralSettings_TestResultsOutputFileNamePostfix = j["GeneralSettings"]["TestResultsOutputFileNamePostfix"].get<std::string>();
  _GeneralSettings_PrintOutStatistics = j["GeneralSettings"]["PrintOutStatistics"].get<bool>();
  _GeneralSettings_KMersSubstrings = j["GeneralSettings"]["KMersSubstrings"].get<bool>();
  _GeneralSettings_BnxSubstrings = j["GeneralSettings"]["BnxSubstrings"].get<bool>();
  _GeneralSettings_BnxReverseMolecules = j["GeneralSettings"]["BnxReverseMolecules"].get<bool>();
  _GeneralSettings_DenoisingDelta = j["GeneralSettings"]["DenoisingDelta"].get<int32_t>();
  _GeneralSettings_BnxRowsToRead = j["GeneralSettings"]["BnxRowsToRead"].get<int32_t>();
  _GeneralSettings_BnxMoleculeMinLabelsCount = j["GeneralSettings"]["BnxMoleculeMinLabelsCount"].get<int32_t>();

  _LengthsEncoderConf_Epsilon = j["LengthsEncoderConf"]["Epsilon"].get<int32_t>();
  _LengthsEncoderConf_LogBase = j["LengthsEncoderConf"]["LogBase"].get<int32_t>();
  _LengthsEncoderConf_Power = j["LengthsEncoderConf"]["Power"].get<float>();
  _LengthsEncoderConf_PolCoeffA = j["LengthsEncoderConf"]["PolCoeffA"].get<float>();
  _LengthsEncoderConf_PolCoeffB = j["LengthsEncoderConf"]["PolCoeffB"].get<float>();
  _LengthsEncoderConf_PolCoeffC = j["LengthsEncoderConf"]["PolCoeffC"].get<float>();
  _LengthsEncoderConf_PolCoeffD = j["LengthsEncoderConf"]["PolCoeffD"].get<float>();
  _LengthsEncoderConf_PolCoeffE = j["LengthsEncoderConf"]["PolCoeffE"].get<float>();

  _GeneralTreeConf_QuestionMark = j["GeneralTreeConf"]["QuestionMark"].get<int32_t>();
  _GeneralTreeConf_Star = j["GeneralTreeConf"]["Star"].get<int32_t>();
  _GeneralTreeConf_ProfilingEnabled = j["GeneralTreeConf"]["ProfilingEnabled"].get<bool>();

  for (auto& chromosome_cnt_iter : j["ChromosomesCnt"])
  {
    _ChromosomesCnt.push_back(chromosome_cnt_iter.get<int16_t>());
  }

  for (auto& test_enable_iter : j["TestEnable"])
  {
    _TestEnable.push_back(test_enable_iter.get<std::string>());
  }

  for (auto& test_def_iter : j["TestDefinitions"])
  {
    auto test_def = test_definition();

    test_def._encode_method = test_def_iter["EncodeMethod"].get<std::string>();
    
    for (auto& depth_iter : test_def_iter["NodeStopCreationDepths"])
    {
      test_def._node_stop_creation_depths.push_back(depth_iter.get<int32_t>());
    }
    
    for (auto& key_type : test_def_iter["TstForestKeyType"])
    {
      test_def._tst_forest_key_type.push_back(key_type.get<std::string>());
    }

    test_def._k_mer_size = test_def_iter["KMerSize"].get<int32_t>();

    _TestDefinitions.push_back(test_def);
  }
}

#endif