#ifndef STATICCONF_H
#define STATICCONF_H

#include <iostream>
#include <string>
#include <vector>
#include <map>

using namespace std;

// enum of row type
enum BnxRowType
{
  Undefined,
  BnxFileHeader,
  Rh,
  RunData,
  ZeroHeader,
  ZeroFormat,
  FirstHeader,
  FirstFormat,
  SecondHeader,
  SecondFormat,
  QualityHeader,
  QualityFormat,
  Data0,
  Data1,
  Quality11,
  Quality12,
  Quality21,
  Quality22
};

// map with mapping from string to BnxRowType enum
static map<string, BnxRowType> StrToBnxRowTypeMap;
// map with names of the enum values for console output purposes
static map<BnxRowType, string> BnxRowTypeNames;

map<string, BnxRowType> GetStrToBnxRowTypeMap()
{
  if (StrToBnxRowTypeMap.empty())
  {
    StrToBnxRowTypeMap = 
    {
      {"# hostname", BnxRowType::BnxFileHeader},
      {"# $", BnxRowType::BnxFileHeader},
      {"# CompileDir", BnxRowType::BnxFileHeader},
      {"# FLAGS", BnxRowType::BnxFileHeader},
      {"# BNX File Version", BnxRowType::BnxFileHeader},
      {"# Label Channels", BnxRowType::BnxFileHeader},
      {"# Nickase Recognition Site 1", BnxRowType::BnxFileHeader},
      {"# Bases per Pixel", BnxRowType::BnxFileHeader},
      {"# Quality Score QX11", BnxRowType::BnxFileHeader},
      {"# Quality Score QX12", BnxRowType::BnxFileHeader},
      {"# Number of Molecules", BnxRowType::BnxFileHeader},
      {"#rh", BnxRowType::Rh},
      {"# Run Data", BnxRowType::RunData},
      {"#0h", BnxRowType::ZeroHeader},
      {"#0f", BnxRowType::ZeroFormat},
      {"#1h", BnxRowType::FirstHeader},
      {"#1f", BnxRowType::FirstFormat},
      {"#2h", BnxRowType::SecondHeader},
      {"#2f", BnxRowType::SecondFormat},
      {"#Qh", BnxRowType::QualityHeader},
      {"#Qf", BnxRowType::QualityFormat},
      {"0", BnxRowType::Data0},
      {"1", BnxRowType::Data1},
      {"QX11", BnxRowType::Quality11},
      {"QX12", BnxRowType::Quality12},
      {"QX21", BnxRowType::Quality21},
      {"QX22", BnxRowType::Quality22}
    };
  }

  return StrToBnxRowTypeMap;
}

map<BnxRowType, string> GetBnxRowTypeNames()
{
  if (BnxRowTypeNames.empty())
  {
    BnxRowTypeNames = 
    {
      {BnxRowType::Undefined, "Undefined"},
      {BnxRowType::BnxFileHeader, "BnxFileHeader"},
      {BnxRowType::Rh, "#rh"},
      {BnxRowType::RunData, "# Run Data"},
      {BnxRowType::ZeroHeader, "#0h"},
      {BnxRowType::ZeroFormat, "#0f"},
      {BnxRowType::FirstHeader, "#1h"},
      {BnxRowType::FirstFormat, "#1f"},
      {BnxRowType::SecondHeader, "#2h"},
      {BnxRowType::SecondFormat, "#2f"},
      {BnxRowType::QualityHeader, "#Qh"},
      {BnxRowType::QualityFormat, "#Qf"},
      {BnxRowType::Data0, "0"},
      {BnxRowType::Data1, "1"},
      {BnxRowType::Quality11, "QX11"},
      {BnxRowType::Quality12, "QX12"},
      {BnxRowType::Quality21, "QX21"},
      {BnxRowType::Quality22, "QX22"}
    };
  }

  return BnxRowTypeNames;
}

#endif