#ifndef _TST_FOREST_TEST_
#define _TST_FOREST_TEST_

#include <filesystem>
#include "bioStructures.h"

using std::endl;
using DT = int32_t;

#include "TernarySearchTree/tst_forest.h"
#include "tst_test_loader.h"
#include "test_utils.h"
#include "SuffixTree/suffix_tree.h"

void tst_forest_test()
{
	vector<double> r;
	vector<SequenceInfo> ri;
	vector<vector<int32_t>> bnx_label_distances;
	vector<int32_t> bnx_ids;

	// load reference genome (i.e. hg38) and molecules from BNX
	load_vectors(r, ri, bnx_label_distances, bnx_ids);

	// BNX data not needed now
	// BNX data are already label distances so only find out max values for encoder
	// vector<DT> bnx_max_label_distances;
	// bnx_max_label_distances.reserve(bnx_label_distances.size());

	// for (auto& bnx_molecule : bnx_label_distances)
	// {
	// 	DT bnx_max_label_distance = 0;
		
	// 	for (auto label_distance : bnx_molecule)
	// 	{
	// 		if (bnx_max_label_distance < label_distance)
	// 		{
	// 			bnx_max_label_distance = label_distance;
	// 		}
	// 	}

	// 	bnx_max_label_distances.push_back(bnx_max_label_distance);
	// }

	// prepare logging file for test results
	std::string test_results_file = _GeneralSettings_PathOutputDir + "/" + _GeneralSettings_TestResultsOutputFileNamePrefix + _GeneralSettings_TestResultsOutputFileNamePostfix;
	if (std::filesystem::exists(test_results_file))
	{
		std::filesystem::remove(test_results_file);
	}
	ofstream test_results_f;
	test_results_f.open(test_results_file);
	
	// loop through chromosomes count (size of input data)
	for (auto& chromosomes_cnt : _ChromosomesCnt)
	{
		// take data from chromosome(s) based on configuration
		int tot_offset = 0;
		for (int i = 0; i < chromosomes_cnt; i++)
		{
			tot_offset += ri[i].length;
		}

		const vector<double> reference_genome_data(r.begin(), r.begin() + tot_offset);
		auto ref_genome_data_size = reference_genome_data.size();

		// calculate distances between the labels of reference genome
		vector<DT> label_distances;
		vector<DT> source_offsets;
		label_distances.reserve(reference_genome_data.size() - 1);
		source_offsets.reserve(reference_genome_data.size() - 1);
		DT max_label_distance;

		calculate_label_distances_and_offsets<DT>(reference_genome_data, label_distances, source_offsets, max_label_distance);

		// print log header messages
		test_results_f << "CHROMOSOMES COUNT: " << chromosomes_cnt << endl;
		test_results_f << "MAX DISTANCE BETWEEN LABELS: " << max_label_distance << endl;
		test_results_f << "TOTAL COUNT OF SOURCE LABELS: " << ref_genome_data_size << endl;
		test_results_f << "COUNT OF DISTANCES: " << label_distances.size() << endl << endl;
		test_results_f << "TEST_ID; TEST_TYPE; RAW_SEARCH_TIME_MS; SEARCH_TIME_TOTAL_MS; SEARCH_PREPARE_TIME_MS; INDEXING_TIME_MS; SIZE_KB" << endl;

		// start loop through the test definitions
		cout << "Running tests with " << chromosomes_cnt << " chromosome(s) (" << ref_genome_data_size << " labels) ... " << endl;
		int16_t test_cnt = 0;
		for (auto& test_def : _TestDefinitions)
		{
			test_results_f << "---------------------------------------------------------" << endl;
			cout << ++test_cnt << " - Test: " << get_test_id(test_def) << "  STARTED ... " << std::flush;

			std::vector<int32_t> encoded_label_distances;
			
			if (test_def._encode_method == "NONE")
			{
				encoded_label_distances = std::vector<int32_t>(label_distances.begin(), label_distances.end());
			}
			else
			{
			 	encoded_label_distances = encode_vector<DT>(test_def._encode_method, label_distances, max_label_distance);
			}

			// alphabet histogram
			// auto alphabet_counts = calc_alphabet_counts_without_max(encoded_label_distances, test_results_f);
			// test_results_f << get_test_id(test_def) << " Alphabet size: " << get_alphabet_size(alphabet_counts) << endl;

			// log_histogram(alphabet_counts, 1000, test_results_f);
			// continue;

			// BNX data not needed now
			// vector<vector<DT>> encoded_bnx_distances;
			// for (int32_t i = 0; i < bnx_label_distances.size(); i++)
			// {
			// 	encoded_bnx_distances.push_back(encode_vector<DT>(test_def._encode_method, bnx_label_distances[i], bnx_max_label_distances[i]));
			// }

			// create k-mers from reference genome
			vector<vector<DT>> reference_k_mers;
			vector<DT> reference_k_mer_offsets; 

			create_k_mers<DT>(test_def._k_mer_size, encoded_label_distances, source_offsets, reference_k_mers, reference_k_mer_offsets);

			// BNX data not needed now
			// create k-mers from BNX molecules
			// vector<vector<vector<DT>>> bnx_k_mers;
			// vector<DT> dummy_offsets;
			// for (auto& bnx_molecule : encoded_bnx_distances)
			// {
			// 	vector<vector<DT>> bnx_molecule_k_mers;
			// 	vector<DT> dummy_output;

			// 	create_k_mers<DT>(test_def._k_mer_size, bnx_molecule, dummy_offsets, bnx_molecule_k_mers, dummy_output);

			// 	bnx_k_mers.push_back(bnx_molecule_k_mers);
			// }

			int64_t search_prep_time;
			int64_t raw_search_time;
			vector<DT> not_found;
			auto found_cnt = 0;

			// ------------------------------------------------------------------------------------------------
			// index the k-mer and their offsets agains source data in TST AVX FOREST
			// ------------------------------------------------------------------------------------------------
			if (is_test_enabled("FOREST"))
			{
			    vector<bool> result;

				for (auto& key_type : test_def._tst_forest_key_type)
				{
					for (auto& node_stop_creation_depth : test_def._node_stop_creation_depths)
					{
						tst_forest forest(node_stop_creation_depth);

						auto start = std::chrono::high_resolution_clock::now();

						if (key_type == "SUM")
						{
							forest.insert(&reference_k_mers, &reference_k_mer_offsets);
						}
						else if (key_type == "32")
						{
							forest.insert_composite_32(&reference_k_mers, &reference_k_mer_offsets);
						}
						else if (key_type == "48")
						{
							forest.insert_composite_48(&reference_k_mers, &reference_k_mer_offsets);
						}
						else
						{
							// not known key type
							throw std::exception();
						}

						auto forest_size = forest.get_tst_forest_size();

						auto end = std::chrono::high_resolution_clock::now();
						auto diff = std::chrono::duration_cast<std::chrono::milliseconds>(end - start);
						auto indexing_time = diff.count();

						// try to search for k-mers in TST AVX FOREST
						vector<vector<DT>> search_vect(reference_k_mers.begin(), reference_k_mers.end());

						if (key_type == "SUM")
						{
							result = forest.search(&search_vect, search_vect.size(), search_prep_time, raw_search_time);
						}
						else if (key_type == "32")
						{
							result = forest.search_composite_32(&search_vect, search_vect.size(), search_prep_time, raw_search_time);
						}
						else if (key_type == "48")
						{
							result = forest.search_composite_48(&search_vect, search_vect.size(), search_prep_time, raw_search_time);
						}
						else
						{
							// not known key type
							throw std::exception();
						}

						auto search_time = search_prep_time + raw_search_time;

						// analyze search results
						not_found.clear();
						found_cnt = 0;
						for (auto i = 0; i < result.size(); i++)
						{
							if (result[i] != 1)
							{
								not_found.push_back(i);
							}
							else
							{
								++found_cnt;
							}
						}

						test_results_f << create_log_message(get_test_id(test_def), "TST AVX FOREST-" + key_type + "-" + 
							std::to_string(node_stop_creation_depth), indexing_time, search_time, search_prep_time, raw_search_time, found_cnt, 
							not_found.size(), forest_size) << endl;
					}
				}
			}

			// ------------------------------------------------------------------------------------------------
			// index the k-mer and their offsets agains source data in TST AVX
			// ------------------------------------------------------------------------------------------------
			if (is_test_enabled("TST"))
			{
				// count all letters => max number of nodes in the TST
				auto keys_size = reference_k_mers.size();
				auto nodes_max_cnt = 0;
				for (auto& k_mer : reference_k_mers)
				{
					for (auto val : k_mer)
					{
						nodes_max_cnt++;
					}
				}

				for (auto& node_stop_creation_depth : test_def._node_stop_creation_depths)
				{
					tst tst_avx(nodes_max_cnt, node_stop_creation_depth);

					auto start = std::chrono::high_resolution_clock::now();

					tst_avx.insert(&reference_k_mers, &reference_k_mer_offsets, keys_size);

					auto end = std::chrono::high_resolution_clock::now();
					auto diff = std::chrono::duration_cast<std::chrono::milliseconds>(end - start);
					auto indexing_time = diff.count();

					tst_avx.clean_up_memory();
					auto arr_size = tst_avx.get_tst_size();

					// try to search for k-mers in TST AVX
					vector<vector<DT>> search_vect(reference_k_mers.begin(), reference_k_mers.end());
					auto search_vect_size = search_vect.size();

					// init vector for results
					std::vector<std::vector<int32_t>> res(search_vect_size);
					for (auto& ret_val_iter : res)
					{
						ret_val_iter.reserve(10);
					}

					start = std::chrono::high_resolution_clock::now();

					tst_avx.search(&search_vect, search_vect_size, res);

					end = std::chrono::high_resolution_clock::now();
					diff = std::chrono::duration_cast<std::chrono::milliseconds>(end - start);
					raw_search_time = diff.count();
					
					search_prep_time = 0;
					auto search_time = search_prep_time + raw_search_time;

					vector<bool> result(reference_k_mers.size());
					for (int i = 0; i < res.size(); i++)
					{
						result[i] = res[i].size() > 0 ? true : false;
					}

					// analyze search results
					not_found.clear();
					found_cnt = 0;
					for (auto i = 0; i < result.size(); i++)
					{
						if (result[i] != 1)
						{
							not_found.push_back(i);
						}
						else
						{
							++found_cnt;
						}
					}

					test_results_f << create_log_message(get_test_id(test_def), "TST AVX-" + std::to_string(node_stop_creation_depth), 
						indexing_time, search_time, search_prep_time, raw_search_time, found_cnt, not_found.size(), arr_size) << endl;
				}
			}

			// ------------------------------------------------------------------------------------------------
			// index k-mers in suffix tree
			// ------------------------------------------------------------------------------------------------
			if (is_test_enabled("GEN_SUFFIX"))
			{
				vector<bool> result;

				SuffixTree<std::vector<int32_t>, int32_t> suffix_tree;

				auto start = std::chrono::high_resolution_clock::now();

				suffix_tree.nodes_cnt = 0;
				for (auto& k_mer : reference_k_mers)
				{
					suffix_tree.insert(k_mer, 0);
				}

				auto end = std::chrono::high_resolution_clock::now();
				auto diff = std::chrono::duration_cast<std::chrono::milliseconds>(end - start);
				auto indexing_time = diff.count();

				result.reserve(reference_k_mers.size());

				vector<vector<DT>> search_vect(reference_k_mers.begin(), reference_k_mers.end());
					
				start = std::chrono::high_resolution_clock::now();

				for (auto& k_mer : search_vect)
				{
					result.push_back(suffix_tree.find_with_positions(k_mer).size() > 0 ? true : false);
				}

				end = std::chrono::high_resolution_clock::now();
				diff = std::chrono::duration_cast<std::chrono::milliseconds>(end - start);
				raw_search_time = diff.count();
				
				search_prep_time = 0;
				auto search_time = search_prep_time + raw_search_time;

				// analyze search results
				not_found.clear();
				found_cnt = 0;
				for (auto i = 0; i < result.size(); i++)
				{
					if (result[i] != 1)
					{
						not_found.push_back(i);
					}
					else
					{
						++found_cnt;
					}
				}

				test_results_f << create_log_message(get_test_id(test_def), "SUFFIX TREE GEN", indexing_time, search_time, search_prep_time, raw_search_time,
					found_cnt, not_found.size(), suffix_tree.get_size_kb()) << endl;
			}

			// ------------------------------------------------------------------------------------------------
			// index encoded label distances in suffix tree (one string)
			// ------------------------------------------------------------------------------------------------
			if (is_test_enabled("SUFFIX"))
			{
				SuffixTree<std::vector<int32_t>, int32_t> suffix_tree_one;

				auto start = std::chrono::high_resolution_clock::now();

				suffix_tree_one.nodes_cnt = 0;
				suffix_tree_one.insert(encoded_label_distances, 0);

				auto end = std::chrono::high_resolution_clock::now();
				auto diff = std::chrono::duration_cast<std::chrono::milliseconds>(end - start);
				auto indexing_time = diff.count();

				vector<bool> result;
				result.reserve(reference_k_mers.size());

				vector<vector<DT>> search_vect(reference_k_mers.begin(), reference_k_mers.end());
					
				start = std::chrono::high_resolution_clock::now();

				for (auto& k_mer : search_vect)
				{
					result.push_back(suffix_tree_one.find_with_positions(k_mer).size() > 0 ? true : false);
				}

				end = std::chrono::high_resolution_clock::now();
				diff = std::chrono::duration_cast<std::chrono::milliseconds>(end - start);
				raw_search_time = diff.count();
				
				search_prep_time = 0;
				auto search_time = search_prep_time + raw_search_time;

				// analyze search results
				not_found.clear();
				found_cnt = 0;
				for (auto i = 0; i < result.size(); i++)
				{
					if (result[i] != 1)
					{
						not_found.push_back(i);
					}
					else
					{
						++found_cnt;
					}
				}

				test_results_f << create_log_message(get_test_id(test_def), "SUFFIX TREE", indexing_time, search_time, search_prep_time, raw_search_time,
					found_cnt, not_found.size(), suffix_tree_one.get_size_kb()) << endl;
			}

			cout << "FINISHED" << endl;
		}
	}

	// close the test results file	
	test_results_f.close();
}

#endif