#include <Loader.h>
#include <Sequencer.h>
#include "BnxParser/bnx.hpp"
#include "BnxParser/runsdata.hpp"

using DT = int32_t;

void load_vectors(vector<double>& r, vector<SequenceInfo>& ri, vector<vector<int32_t>>& bnx_pos, vector<int32_t>& bnx_ids)
{
    // parse FASTA and save it to files if output files don't exist
    Sequencer<double> sequencer;
    auto r_file_name = _GeneralSettings_PathOutputDir + "/" + _GeneralSettings_OutputRFileName;
    auto ri_file_name = _GeneralSettings_PathOutputDir + "/" + _GeneralSettings_OutputRiFileName;
	
    if (!(std::filesystem::exists(r_file_name) && std::filesystem::exists(ri_file_name)))
	{
		r.reserve(1 << 20);
		ri.reserve(1 << 20);

		cout << "Loading ... " << std::flush << (Loader::parseMarkersFromFasta(_GeneralSettings_PathRefGenomeFasta, _GeneralSettings_PathOutputDir, _GeneralSettings_MarkerSequence, r, ri) ? "OK" : "FAILED") << endl;
		printf("%-20s:\t%lu\n", "No sequences", static_cast<unsigned long>(ri.size()));
		sequencer.denoiseSequences1(r, ri, _GeneralSettings_DenoisingDelta);

		cout << "Serialization to " << r_file_name << " ... " << (pg::stdvectorutils::serialize(r_file_name.c_str(), r) ? "OK" : "FAILED") << endl;
		cout << "Serialization to " << ri_file_name << " ... " << (pg::stdvectorutils::serialize(ri_file_name.c_str(), ri) ? "OK" : "FAILED") << endl;
	}

	// fill r and ri vectors from files if empty
	if ((r.empty() || ri.empty()) && std::filesystem::exists(r_file_name) && std::filesystem::exists(ri_file_name))
	{
        r.clear();
        ri.clear();

		r.reserve(1 << 20);
		ri.reserve(1 << 20);

		cout << "Deserialization from " << r_file_name << " ... " << (pg::stdvectorutils::deSerialize(r_file_name.c_str(), r) ? "OK" : "FAILED") << endl;
		cout << "Deserialization from " << ri_file_name << " ... " << (pg::stdvectorutils::deSerialize(ri_file_name.c_str(), ri) ? "OK" : "FAILED") << endl;
	}

    // read BNX and save it to files if output files don't exist
    auto bnx_pos_file_name = _GeneralSettings_PathOutputDir + "/" + _GeneralSettings_OutputPosBnxFileName;
    auto bnx_ids_file_name = _GeneralSettings_PathOutputDir + "/" + _GeneralSettings_OutputMolIdsBnxFileName;

    if (!(std::filesystem::exists(bnx_pos_file_name) && std::filesystem::exists(bnx_ids_file_name)))
    {
        auto* bnxFile = new Bnx(_GeneralSettings_PathBnxFile);
        cout << "Loading BNX ... " << std::flush << (bnxFile->Fill(_GeneralSettings_BnxRowsToRead, _GeneralSettings_PrintOutStatistics) ? "OK" : "FALSE") << endl;

        auto* runs_data = new RunsData<DT>();
	      runs_data->FillDataFromBnx(bnxFile, _GeneralSettings_BnxReverseMolecules, _GeneralSettings_BnxMoleculeMinLabelsCount, _GeneralSettings_PrintOutStatistics);

        auto run_it = runs_data->_data.begin();
        while (run_it != runs_data->_data.end())
        {
            auto* run = run_it->second;

            for (auto const& molecule : run->_molecules)
            {
                vector<int32_t> vect;
                for (int _labelDistance : molecule->_labelDistances)
                {
                    vect.push_back(_labelDistance);
                }

                bnx_pos.push_back(vect);
                bnx_ids.push_back(molecule->_moleculeId);
            }

            ++run_it;
        }

        cout << "Serialization to " << bnx_pos_file_name << " ... " << (pg::stdvectorutils::serialize(bnx_pos_file_name.c_str(), bnx_pos) ? "OK" : "FAILED") << endl;
		    cout << "Serialization to " << bnx_ids_file_name << " ... " << (pg::stdvectorutils::serialize(bnx_ids_file_name.c_str(), bnx_ids) ? "OK" : "FAILED") << endl;
    }

    // fill r and ri vectors from files if empty
	if ((bnx_pos.empty() || bnx_ids.empty()) && std::filesystem::exists(bnx_pos_file_name) && std::filesystem::exists(bnx_ids_file_name))
	{
        bnx_pos.clear();
        bnx_ids.clear();

		bnx_pos.reserve(1 << 20);
		bnx_ids.reserve(1 << 20);

		cout << "Deserialization from " << bnx_pos_file_name << " ... " << (pg::stdvectorutils::deSerialize(bnx_pos_file_name.c_str(), bnx_pos) ? "OK" : "FAILED") << endl;
		cout << "Deserialization from " << bnx_ids_file_name << " ... " << (pg::stdvectorutils::deSerialize(bnx_ids_file_name.c_str(), bnx_ids) ? "OK" : "FAILED") << endl;
	}
}