set(compile_tests ON)

file(GLOB MY_APP_SOURCE_FILES
    ${MY_SRC_DIR}/*.h
    ${MY_SRC_DIR}/*.cpp
)

file(GLOB MY_TEST_SOURCE_FILES RELATIVE ${MY_TEST_DIR}
    ${MY_TEST_DIR}/*.cpp
)

set(production_sources ${MY_APP_SOURCE_FILES})
# list(REMOVE_ITEM production_sources ${MY_SRC_DIR}/main.cpp)
print_glob("production_sources")

set(testing_sources ${MY_APP_SOURCE_FILES})
print_glob("testing_sources")

find_package(OpenMP)
if (OPENMP_FOUND)
    set (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} ${OpenMP_C_FLAGS}")
    set (CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} ${OpenMP_CXX_FLAGS}")
    set (CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} ${OpenMP_EXE_LINKER_FLAGS}")
endif()

if (compile_tests)

    include_directories(BEFORE 
        ${MY_COMMON_DIR}/JSON/
        ${MY_COMMON_DIR}/pg/
        ${MY_SRC_DIR}
    )

    link_directories(BEFORE "stdc++fs" )

    foreach( testsourcefile ${MY_TEST_SOURCE_FILES} )
        string( REPLACE ".cpp" "" testname ${testsourcefile} )  # I used a simple string replacement, to cut off .cpp.
        add_executable( ${testname} ${MY_TEST_DIR}/${testsourcefile} ${MY_APP_SOURCE_FILES})

        target_include_directories(${testname} PRIVATE
            ${MY_COMMON_DIR}/JSON/
            ${MY_COMMON_DIR}/pg/
            ${MY_SRC_DIR}
        )

        target_link_libraries(${testname} PRIVATE "stdc++fs")
        target_link_libraries(${testname} PRIVATE OpenMP::OpenMP_CXX)
        set_target_properties(${testname} PROPERTIES PREFIX "")            #clears the lib prefix for library name
        # set_target_properties(${testname} PROPERTIES RUNTIME_OUTPUT_DIRECTORY ${MY_ROOT_SRC_DIR}/bin )
    
    endforeach( testsourcefile ${MY_TEST_SOURCE_FILES} )

else ()
    add_library(bioutils STATIC ${production_sources})

    target_include_directories(bioutils PRIVATE 
        ${MY_COMMON_DIR}/JSON/
        ${MY_COMMON_DIR}/pg/
        ${MY_SRC_DIR}
    )

    target_link_libraries(bioutils PRIVATE "stdc++fs")
    target_link_libraries(bioutils PRIVATE OpenMP::OpenMP_CXX)
    set_target_properties(bioutils PROPERTIES PREFIX "")            #clears the lib prefix for library name
endif()


# add_custom_command(
#     TARGET app
#     POST_BUILD
#     COMMAND ${CMAKE_COMMAND} -E copy ${MY_SRC_DIR}/config.json $<TARGET_FILE_DIR:app>/config.json
#     COMMENT "Copying to output directory"
# )