#include "Loader.h"

#include <sstream>
#include <unordered_set>

// #include <tr2/type_traits>

#include <utils/fileUtils.h>
#include <ProgressBar.h>
#include <TokenizerSimple.h>

using namespace std;
using namespace pg::fileutils;
using std::string_view;

bool Loader::parseIDSfromBNX(const string& fileName, vector<std::uint32_t>& ids)
{
	try 
    {
        char* buffer = nullptr;
        std::streampos fileSize;

        if (!pg::fileutils::getBuffer(fileName, buffer, fileSize)) return false;

		string_view sw(buffer);
		string_view::size_type start = 0;
		string_view::size_type pos = sw.find('\n', start);      //pos = sw.find_first_of('\n', start);

		std::uint32_t tmpIdx=0;
		while (pos != string_view::npos)
		{
			if (pos <= start) break;

			if (*(buffer + start) == '0')
			{
				pg::parsing::parseValue(buffer + start + 2, buffer + pos, tmpIdx);	//+2 skips: '0' + separator
				ids.emplace_back(tmpIdx);
			} 
			start = pos + 1;
			pos = sw.find('\n', start);                             //pos = sw.find_first_of('\n', start);
		}
		ids.shrink_to_fit();

        ::operator delete(buffer);
        return true;    
    }
    catch (const std::exception& e)
    {
        std::cerr << e.what() << std::endl;
    }
    return false;	
}

bool Loader::loadCMAPSequences(const char* fileName, vector<float>& rawData, vector<SequenceInfo>& sequencesInfo)
{
	stringstream f;
	if (!getStringstream(string(fileName), f))
	{
		return false;
	}

	uint32_t id = 0;
	uint32_t offset = 0;
	float data;

	f.seekg(0, ios::end);
	float streamLength = static_cast<float>(f.tellg());
	f.seekg(0, ios::beg);
	float currentPos = 0.0f;
	ProgressBar progress0{std::clog, 70u, "Loading: "};

	while (f >> id)
	{
		sequencesInfo.push_back(SequenceInfo());
		SequenceInfo& si = sequencesInfo.back();
		f >> si.length;
		si.id = id;
		si.offset = offset;
		offset += si.length;

		//f.ignore(5, ':');

		for (unsigned int i=0; i< si.length; i++)
		{
			f >> data;
			rawData.emplace_back(data);
		}

		currentPos = static_cast<float>(f.tellg());
		progress0.write(currentPos/streamLength);
	}

	rawData.shrink_to_fit();
	sequencesInfo.shrink_to_fit();
    return true;	
}

bool Loader::loadFinalAlignments(const char* fileName, vector<AlignmentData>& alignmentData)
{
	stringstream f;
	if (!getStringstream(string(fileName), f))
	{
		return false;
	}
	
	f.seekg(0, ios::end);
	float streamLength = static_cast<float>(f.tellg());
	f.seekg(0, ios::beg);
	float currentPos = 0.0f;
	ProgressBar progress0{std::clog, 70u, "Loading: "};

	uint32_t id = 0;
	while (f >> id)
	{
		alignmentData.push_back({id});
		AlignmentData& ad = alignmentData.back();

		f >> ad.cid;
		f >> ad.rsid;
		f >> ad.sStart;
		f >> ad.sEnd;
		f >> ad.rsStart;
		f >> ad.rsEnd;

		currentPos = static_cast<float>(f.tellg());
		progress0.write(currentPos/streamLength);
	}

	alignmentData.shrink_to_fit();
    return true;	
}