#pragma once

#include <cstdint>
#include <string>
#include <fstream>

namespace pg::serializing
{
	//This templates will be specialized below
    template<typename T> constexpr void serialize(const T& value, std::ofstream& s) {}
    template<typename T> constexpr void deserialize(T& value, std::ifstream& s) {}

    ///====================================================================================================
    ///@brief STRING - specialization for serialize and deserialize template methods. 
    ///====================================================================================================
    template<> inline void serialize(const std::string& value, std::ofstream& s) 
    {
        std::string::size_type size = value.size();
        s.write(reinterpret_cast<const char*>(&size),  sizeof(std::string::size_type));
        s.write(value.c_str(), size);
    }

    template<> inline void deserialize(std::string& value, std::ifstream& s) 
    {
        std::string::size_type size = value.size();
        s.read(reinterpret_cast<char*>(&size),  sizeof(std::string::size_type));
        if (size > 0)
        {   
            value.resize(size);
            s.read(reinterpret_cast<char*>(&value[0]), size);
        }
    }
}