#pragma once

namespace pg::sfinae
{
	/// Defines a "has_member_member_name" class template
	/// This template can be used to check if its "T" argument
	/// has a data or function member called "member_name"
	#define define_has_member(member_name)                                         \
		template <typename T>                                                      \
		class has_member_##member_name                                             \
		{                                                                          \
			typedef char yes_type;                                                 \
			typedef long no_type;                                                  \
			template <typename U> static yes_type test(decltype(&U::member_name)); \
			template <typename U> static no_type  test(...);                       \
		public:                                                                    \
			static constexpr bool value = sizeof(test<T>(0)) == sizeof(yes_type);  \
		}

	/// Shorthand for testing if "class_" has a member called "member_name"
	///
	/// @note "define_has_member(member_name)" must be used
	///       before calling "has_member(class_, member_name)"
	#define has_member(class_, member_name)  has_member_##member_name<class_>::value
}
